from django.shortcuts import render,redirect
from django.contrib.auth.decorators import login_required,user_passes_test
from django.utils.decorators import method_decorator
#from mainApp.serializers import *
#from mainApp.models import *
from rest_framework.response import Response
from rest_framework import status
from rest_framework.views import APIView
#import json , requests
from django.conf import settings as stg
from datetime import datetime,date,timedelta
from django.db.models import Count,Sum
import json
from django.utils import timezone
from itertools import chain # Join Multiple Querysets From Different Base Models Django
from django.db.models import Q
from clmainApp.utils import getClinicId
from .models import *
from .serializers import *
from clmainApp.utils import getBranchId
# Create your views here.
###### sessions   ########
class SessionsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_sessions'), login_url='/main/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        all_sessions =Session.objects.filter(branch_id=branchId)
        all_sessions_serialized=SessionSerializer(all_sessions,many=True)
        return render(request,"Branch/brsessionApp/br_sessions.html",{"all_sessions":all_sessions_serialized.data,'activePage':'MedicalSessions'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_session'), login_url='/main/login/'))    
    def post(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        data = request.data
        data._mutable = True
        data.update({"created_by":request.user.id,"branch":branchId})
        serializedObj= SessionSerializer(data=data)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class SessionApi(APIView):
    def get_object(self,pk):
        try:
            return Session.objects.get(pk=pk)
        except Session.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_sessions'), login_url='/main/login/'))        
    def get(self,request,pk): 
        sessionObj=self.get_object(pk)
        if sessionObj :
            serializeobj= SessionSerializer(sessionObj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_session'), login_url='/main/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        #request.data._mutable = True
        serializedObj= SessionSerializer(obj,data=request.data,partial=True)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)
            

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_session'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

class CurrentSessionsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_currentsessions'), login_url='/main/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        user = request.user
        if user.is_superuser :patient_sessions =PatientSessionItem.objects.filter(entry_status__in=["1I","2W"],session__patient__branch_id=branchId).order_by("entry_status")
        else :
            patient_sessions =PatientSessionItem.objects.filter(entry_status__in=["1I","2W"],
                doctor_id=user.id,session__patient__branch_id=branchId).order_by("entry_status")
        #all_patient_visits =PatientVisit.objects.exclude(visit_status="4F").order_by("visit_status","-entry_datetime")
        serializer=PatientSessionItemMiniSerializer(patient_sessions,many=True)
        return render(request,"Branch/brsessionApp/current_sessions.html",
        {"patient_sessions":serializer.data,'activePage':'CurrentSessions'}
        )

from django.db.models import Q
#from django.db.models import F, ExpressionWrapper, IntegerField
##from django.db.models.functions import Mod,RowNumber
#from django.db import models
from itertools import groupby

class ToRateSessionsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_toRateSessions'), login_url='/main/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        #user = request.user
        #import pdb
        #pdb.set_trace()
        # Query for PatientSessionItem objects where:
        # - psi_order is even (i.e., divisible by 2) and greater than or equal to 2
        # - The related PatientSession's is_package is True
        # Annotate the result with the modulo of psi_order, then filter where psi_order % 2 == 0
        timezone_now_today = timezone.now().date()
        ssn_patients_today = PatientSessionItem.objects.filter(
            #evaluator_note=None,
            session_day=timezone_now_today,
            session__patient__branch_id=branchId,
        )
        #ssn_today_ids = ssn_patients_today.values_list('id',flat=True)
        ssn_patients_today_ids = ssn_patients_today.values_list('session__patient_id',flat=True)
        # Step 1: Fetch all PatientSessionItem objects ordered by id
        all_sessions = PatientSessionItem.objects.filter( 
            #~Q(entry_status__in=["5A", "6C"]),# Negate the condition to exclude these statuses
            session__patient_id__in=ssn_patients_today_ids,  # Filter by patient ids 
        ).values('id',"session__patient_id").order_by('session__patient_id')  # Order by id
        # Group sessions by patient
        grouped_sessions = groupby(all_sessions, key=lambda x: x["session__patient_id"])
        # Create a dictionary to store even sessions for each patient
        #patient_even_sessions = {}
        even_sessions_patients = []
        for patient_id, sessions in grouped_sessions:
            # Enumerate sessions for the current patient
            sessions_ids = [session["id"] for session in sessions]
            sessions_ids.sort()
            enumerated_sessions = [(index + 1, session_id) for index, session_id in enumerate(sessions_ids)]
            # Filter out even-numbered sessions based on their enumeration
            even_sessions_ids = [session_id for index, session_id in enumerated_sessions if index % 2 == 0]
            even_sessions_patients.extend(even_sessions_ids)
            #patient_even_sessions[patient_id] = even_sessions
        even_sessions = ssn_patients_today.filter(id__in=even_sessions_patients)
        #pdb.set_trace()
        #psi_order_list = all_sessions.values_list('psi_order',flat=True)
        #print(len(psi_order_list))
        # Step 2: Filter to get only even sessions using enumerate() in Python
        # even sessions among all 
        #even_sessions = [session for index, session in enumerate(all_sessions, start=1) if index % 2 == 0]
        #for e_ssn in even_sessions : print(e_ssn.psi_order,"---",e_ssn.session.is_package)
        #print(len(even_sessions))
        #even_sessions = PatientSessionItem.objects.filter( session__patient_id__in=patients_today_ids ).annotate( row_number=RowNumber(order_by=F('id').asc()) )

        # even_sessions = PatientSessionItem.objects.filter(
        #     session__patient_id__in=patients_today_ids  # Filter by the specific patient instance
        # ).annotate(
        #     row_number=RowNumber(order_by=F('id').asc())  # Annotate row numbers based on id
        # ).filter(
        #     row_number__mod=2,  # Get even row numbers
        # ).order_by('id')  # Order by id
        #pdb.set_trace()
        #even_patient_session_items
        # patient_sessions = PatientSessionItem.objects.annotate(
        # mod_psi_order=Mod(F('psi_order'), 2)
        # ).filter(
        #     evaluator_note=None,
        #     #session_day=timezone_now_today,
        #     mod_psi_order=0,  # psi_order is even
        #     psi_order__gte=2,  # psi_order is greater than or equal to 2
        #     session__is_package=True  # session's is_package is True
        # )
        

        # # Step 1: Use a window function to assign a row number based on id ordering
        # even_sessions = PatientSessionItem.objects.filter(
        #     session__patient=patient_instance  # Filter by the specific patient instance
        # ).annotate(
        #     row_number=RowNumber(order_by=F('id').asc())  # Annotate row numbers based on id
        # ).filter(
        #     row_number__mod=2,  # Get even row numbers
        # ).order_by('id')  # Order by id
        #patient_sessions =PatientSessionItem.objects.filter(entry_status__in=["1I","2W"]
                        #,session__patient__branch_id=branchId).order_by("entry_status")
        #all_patient_visits =PatientVisit.objects.exclude(visit_status="4F").order_by("visit_status","-entry_datetime")
        serializer=PatientSessionItemMiniSerializer(even_sessions,many=True)
        return render(request,"Branch/brsessionApp/to_rate_sessions.html",
        {"patient_sessions":serializer.data,'activePage':'ToRateSessions'}
        )
def checkIfUserIsBranchManger(user):
    user_groups = user.groups.filter(is_branch_manager=True).count()
    return (True if (user_groups > 0 or user.is_superuser)  else False)

class DrTodaySessionsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_drTodaySessions'), login_url='/main/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        user = request.user
        #all_patient_visits =PatientVisit.objects.exclude(visit_status="4F").order_by("visit_status","-entry_datetime")
        #serializer=PatientSessionItemMiniSerializer(patient_sessions,many=True)
        timezone_now_today = timezone.now().date()
        tommorrow_date = timezone_now_today + timedelta(days=1)
        if user.is_superuser :patient_sessions =PatientSessionItem.objects.filter(session_day__in=[timezone_now_today,tommorrow_date],session__patient__branch_id=branchId).order_by('session_day','hour_slot__order')
        else :
            patient_sessions =PatientSessionItem.objects.filter(doctor_id=user.id,session__patient__branch_id=branchId).order_by("entry_status").order_by('session_day','hour_slot__order')

        today_sessions = PatientSessionItemCustomSerializer(patient_sessions.filter(session_day=timezone_now_today),many=True)
        tommorrow_sessions = PatientSessionItemCustomSerializer(patient_sessions.filter(session_day=tommorrow_date),many=True)

        return render(request,"Branch/brsessionApp/dr_today_sessions.html",
        {"today_sessions":today_sessions.data,"today_date":timezone_now_today,
         'tommorrow_sessions':tommorrow_sessions.data,"tommorrow_date":tommorrow_date,
         "isBranchManger":checkIfUserIsBranchManger(user),
         'activePage':'DrTodaySessions'}
        )
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_drTodaySessions'), login_url='/main/login/'))
    def post(self,request):
        branchId = getBranchId(request)
        #clinicId = getClinicId(request)
        if not branchId : return Response(status=status.HTTP_400_BAD_REQUEST)
        data = request.data
        session_day = data["session_day"]
        patient_sessions =PatientSessionItem.objects.filter(session_day = session_day,session__patient__branch_id=branchId).order_by("entry_status").order_by('session_day','hour_slot__order')
        serializer = PatientSessionItemCustomSerializer(patient_sessions,many=True)
        return Response(serializer.data , status=status.HTTP_200_OK)
        
class PatientSessionsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_ptSessions'), login_url='/auth/login/'))
    def get(self,request):
        # if dr  , get current branch patients
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        all_patients =Patient.objects.filter(
            branch_id=branchId,code__gt=0).values("id","fullname","rfid","code","number_1")
        return render(request,"Branch/brsessionApp/pt_sessions.html",{
            "all_patients":all_patients,'activePage':'PatientSessions'})
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_patient'), login_url='/auth/login/'))    
    def post(self,request):
        branchId = getBranchId(request)
        #clinicId = getClinicId(request)
        if not branchId : return Response(status=status.HTTP_400_BAD_REQUEST)
        data = request.data
        data._mutable = True
        patient_sessions = PatientSession.objects.filter(patient_id=data["patient_id"]).order_by('-id')
        serializer = PatientSessionSerializer(patient_sessions,many=True)
        return Response(serializer.data , status=status.HTTP_200_OK)
    
from brpatientApp.models import PatientInfo
class PatientSessionItemsApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_ptSessions'), login_url='/auth/login/'))
    def get(self,request,pt_session_id):
        session_items =PatientSessionItem.objects.filter(session_id=pt_session_id)
        serializer = PatientSessionItemCustomSerializer(session_items,many=True)
        return Response(serializer.data,status=status.HTTP_200_OK)

    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_ptSessions'), login_url='/auth/login/'))
    def post(self,request,pt_session_id):#cancel session items
        data = request.data
        patient_id = data["patient_id"]
        psi_ids = data.getlist('psi_ids[]')#هيبقى فى checkboxes قدام الجلسات اللى لسه جديد نعمل زرار الغاء الجلسات 
        try : 
            psi_items = PatientSessionItem.objects.filter(id__in=psi_ids)
            psi_items.update(session_status_id='6CL',entry_status="6C")
            objs_updated_count = PatientInfo.objects.filter(patient_id=patient_id,).update(marketing_code='CANCEL')
            if objs_updated_count == 0 : 
                user_id = request.user.id
                PatientInfo.objects.create(patient_id=patient_id,last_update_by_id=user_id,marketing_code='CANCEL') 
            session_ids = psi_items.values_list("session_id", flat=True)
            pt_sessions = PatientSession.objects.filter(id__in=session_ids)
            for pt_session in pt_sessions:
                sessions_remain = pt_session.sessions_remain 
                pt_session.sessions_remain = sessions_remain - 1 
                pt_session.save()
            return Response(status=status.HTTP_200_OK)#لما التعديل يخلص هنكتب تم الحفظ بنجاح ونخفى الجدول اللى تحت اللى ظهر فيه تفاصيل الجلسات 
        except Exception as err_msg: 
            return Response(data ="{}".format(str(err_msg)),status=status.HTTP_400_BAD_REQUEST)

class PatientSessionApi(APIView):
    def get_object(self,pk):
        try:
            return PatientSession.objects.get(pk=pk)
        except PatientSession.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_ptSessions'), login_url='/main/login/'))        
    def get(self,request,session_pk):
        #context={'activePage':'PatientSessions'}
        context={}
        patient_sessionObj=self.get_object(session_pk)
        if patient_sessionObj :
            context.update({"pS":PatientSessionSerializer(patient_sessionObj).data})
            p_s_items=PatientSessionItem.objects.filter(session=patient_sessionObj).order_by("psi_order")
            psi_inside_finish =p_s_items.filter(entry_status__in=["1I","4F"]).order_by("-entry_datetime")
            p_s_items_dates_slotes =PtSsnItemDaySlotSerializer(p_s_items,many=True)
            psi_inside_finish_serialized =PtSsnItemCustomSerializer(psi_inside_finish,many=True)
            context.update({"p_s_items_dates_slotes":p_s_items_dates_slotes.data})
            context.update({"psi_inside_finish":psi_inside_finish_serialized.data})
            # p_s_ex_items=PsiExrszItem.objects.filter(psi=patient_sessionObj)
            # p_s_ex_items_serialized =PsiExrszItemMiniSerializer(p_s_ex_items,many=True)
            # context.update({"p_s_ex_items":p_s_ex_items_serialized.data})
            context.update({'activePage':'CurrentSessions'})
        return render(request,"Branch/brsessionApp/session.html",context,)
from rcpsessionApp.serializers import PatientSessionItemCustomSerializer
class PtSsnItemApi(APIView):
    def get_object(self,pk):
        try:
            return PatientSessionItem.objects.get(pk=pk)
        except PatientSessionItem.DoesNotExist:
            return None    
    def post(self,request,psi_pk):
        obj=self.get_object(psi_pk)
        if obj :
            data = request.data
            data._mutable = True
            serializer= PatientSessionItemCustomSerializer(obj,data,partial=True)
            if serializer.is_valid():
                serializer.save()
                return Response(serializer.data , status=status.HTTP_200_OK)
            else :return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)
        else : return Response(status=status.HTTP_400_BAD_REQUEST)

    def put(self,request,psi_pk):
        obj=self.get_object(psi_pk)
        if obj :
            obj.is_confirmed = True
            obj.confirmed_by = request.user
            obj.confirmed_at = timezone.now()
            obj.save()
            #لما يرجع 200 نخلى الايقونة صح بدل زرار 
            #obj.doctor = request.user
            return Response(status=status.HTTP_200_OK)
        else :
            return Response(status=status.HTTP_400_BAD_REQUEST)
        
class PsiEvaluationNotesApi(APIView):
    def get_object(self,pk):
        try:
            return PatientSessionItem.objects.get(pk=pk)
        except PatientSessionItem.DoesNotExist:
            return None    

    def put(self,request,psi_pk):
        obj=self.get_object(psi_pk)
        #import pdb
        #pdb.set_trace()
        data = request.data
        if obj :
            obj.evaluated_by = request.user
            obj.evaluated_at = timezone.now()
            obj.evaluator_note = data["evaluator_note"]
            obj.save()
            return Response({"evaluator_name":obj.evaluated_by.name},status=status.HTTP_200_OK)
        else :
            return Response(status=status.HTTP_400_BAD_REQUEST)
        
class PsiAssessmentsApi(APIView):
    
    def post(self,request):
        data = request.data 
        selected_sessions = data.getlist('selected_sessions[]')
        data._mutable = True
        assessment_choice = data['assessment_choice']
        assessment_code = data['assessment_code']
        data.update({"created_by":request.user.id,"created_at":timezone.now(),"sessions_count":len(selected_sessions)})
        serializer = PsiAssessmentSerializer(data=data)
        if serializer.is_valid():
            serializer.save()
            psi_assessment_id = serializer.data["id"]
            if assessment_code == "P":
                PatientSessionItem.objects.filter(id__in=selected_sessions).update(psi_assessment_choice=assessment_choice,assessment_id=psi_assessment_id)
            elif assessment_code =="S":
                PatientSessionItem.objects.filter(id__in=selected_sessions).update(psi_assessment_choice_s=assessment_choice,assessment_s_id=psi_assessment_id)
            return Response(status=status.HTTP_201_CREATED)
        else :return Response(serializer.errors,status=status.HTTP_400_BAD_REQUEST)

    def get_object(self,pk):
        try:
            return PsiAssessment.objects.get(pk=pk)
        except PsiAssessment.DoesNotExist:
            return None   
    def delete(self,request):
        data = request.data
        assessment_id = data["obj_id"]
        obj=self.get_object(assessment_id)
        #print(obj.assessment_code)
        if obj :
            try:
                if obj.assessment_code == "P":
                    PatientSessionItem.objects.filter(assessment_id=assessment_id).update(psi_assessment_choice="UD",assessment_id=None)
                elif obj.assessment_code == "S":
                    PatientSessionItem.objects.filter(assessment_s_id=assessment_id).update(psi_assessment_choice_s="UD",assessment_s_id=None)
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)